importFrom("methods", "as")
useDynLib(tmvtnorm, .registration = TRUE)
import(stats)
import(utils)
import(mvtnorm)
import(stats4)
import(gmm)
import(Matrix)
export(ptmvnorm)
export(rtmvnorm)
export(rtmvnorm2)
export(rtmvnorm.sparseMatrix)
export(dtmvnorm)
export(dtmvnorm.marginal)
export(dtmvnorm.marginal2)
export(qtmvnorm.marginal)
export(ptmvnorm.marginal)
export(mtmvnorm)
export(dtmvt)
export(rtmvt)
export(ptmvt)
export(ptmvt.marginal)
export(mle.tmvnorm)
export(gmm.tmvnorm)