# tmvtnorm

[![CRAN](http://www.r-pkg.org/badges/version/tmvtnorm)](https://cran.r-project.org/package=tmvtnorm)

### tmvtnorm: A package for the Truncated Multivariate Normal Distribution

This package contains a number of useful methods for the truncated multivariate normal distribution. It considers
random number generation with rejection and Gibbs sampling, computation of marginal densities as well as computation of the mean and 
covariance of the truncated variables.

For a more detailed introduction, see this RJournal (2010) paper [tmvtnorm: A Package for the Truncated Multivariate Normal Distribution](https://doi.org/10.32614/RJ-2010-005).

